@@ -1,9 +1,10 @@
 #!/usr/bin/env python
 
 import datetime
-from cStringIO import StringIO
 import itertools
 
+import six
+
 from csvkit import CSVKitReader, CSVKitWriter
 from csvkit import sniffer
 from csvkit import typeinference
@@ -31,7 +32,7 @@ class Column(list):
             t = normal_type
             data = l
         elif not infer_types:
-            t = unicode
+            t = six.text_type 
             data = l
         else:
             t, data = typeinference.normalize_column_type(l, blanks_as_nulls=blanks_as_nulls)
@@ -48,13 +49,19 @@ class Column(list):
         """
         Stringify a description of this column.
         """
-        return u'%3i: %s (%s)' % (self.order, self.name, self.type)
+        return '%3i: %s (%s)' % (self.order, self.name, self.type)
 
     def __getitem__(self, key):
         """
         Return null for keys beyond the range of the column. This allows for columns to be of uneven length and still be merged into rows cleanly.
         """
-        if key >= len(self):
+        l = len(self)
+
+        if isinstance(key, slice):
+            indices = six.moves.range(*key.indices(l))
+            return [(list.__getitem__(self, i) if i < l else None) for i in indices]
+
+        if key >= l:
             return None
 
         return list.__getitem__(self, key)
@@ -73,7 +80,7 @@ class Column(list):
         """
         l = 0
 
-        if self.type == unicode:
+        if self.type == six.text_type:
             l = max([len(d) if d else 0 for d in self])
 
             if self.has_nulls():
@@ -99,7 +106,7 @@ class Table(list):
         """
         Stringify a description of all columns in this table.
         """
-        return u'\n'.join([unicode(c) for c in self])
+        return '\n'.join([six.text_type(c) for c in self])
 
     def _reindex_columns(self):
         """
@@ -199,7 +206,7 @@ class Table(list):
         elif snifflimit > 0:
             kwargs['dialect'] = sniffer.sniff_dialect(contents[:snifflimit])
 
-        f = StringIO(contents)
+        f = six.StringIO(contents)
         rows = CSVKitReader(f, **kwargs)
 
         if no_header_row:
@@ -207,13 +214,14 @@ class Table(list):
             row = next(rows) 
 
             headers = make_default_headers(len(row))
-            column_ids = range(len(row))
+            column_ids = parse_column_identifiers(column_ids, headers, zero_based)
+            headers = [headers[c] for c in column_ids]
             data_columns = [[] for c in headers]
 
             # Put row back on top
             rows = itertools.chain([row], rows)
         else:
-            headers = rows.next()
+            headers = next(rows)
             
             if column_ids:
                 column_ids = parse_column_identifiers(column_ids, headers, zero_based)
@@ -250,14 +258,14 @@ class Table(list):
             for c in self:
                 # Stringify datetimes, dates, and times
                 if c.type in [datetime.datetime, datetime.date, datetime.time]:
-                    out_columns.append([unicode(v.isoformat()) if v != None else None for v in c])
+                    out_columns.append([six.text_type(v.isoformat()) if v != None else None for v in c])
                 else:
                     out_columns.append(c)
 
             # Convert columns to rows 
-            return zip(*out_columns)
+            return list(zip(*out_columns))
         else:
-            return zip(*self)
+            return list(zip(*self))
 
     def to_csv(self, output, **kwargs):
         """
